/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.example.ackNackModel.components;

import ikr.simlib.distributions.continuous.ConstantDistribution;
import ikr.simlib.entities.Entity;
import ikr.simlib.entities.phases.DInfiniteServer;
import ikr.simlib.entities.phases.InfiniteServer;
import ikr.simlib.entities.phases.Phase;
import ikr.simlib.entities.phases.StdPhase;
import ikr.simlib.entities.queues.QueuingEntity;
import ikr.simlib.events.time.Duration;
import ikr.simlib.model.SimNode;
import ikr.simlib.parameters.Parameters;
import ikr.simlib.ports.input.InputPort;
import ikr.simlib.ports.output.OutputPort;

public class Channel extends Entity {

   private final QueuingEntity queue;
   private final Phase phase;
   private final InfiniteServer delay;

   public Channel(SimNode ownNode, Parameters pars) {
      super(ownNode);

      this.queue = QueuingEntity.createUnboundedFIFOQueue(this.simNode.createChildNode("Queue"));
      final double processingTime = pars.get(this.simNode, "ProcessingTime").asDouble();
      // this.phase = new StdPhase(new NegExpDistribution(processingTime),
      // this.simNode.createChildNode("Phase"));
      this.phase = new StdPhase(new ConstantDistribution(processingTime), this.simNode.createChildNode("Phase"));
      final double delayTime = pars.get(this.simNode, "Delay").asDouble();
      this.delay = new DInfiniteServer(Duration.fromSeconds(delayTime), this.simNode.createChildNode("Delay"));

      this.queue.getOutput().connect(this.phase.getInput());
      this.phase.getOutput().connect(this.delay.getInput());
   }

   public InputPort getInput() {
      return this.queue.getInput();
   }

   public OutputPort getOutput() {
      return this.delay.getOutput();
   }
}
