/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.example.ackNackModel.components;

import ikr.simlib.entities.Entity;
import ikr.simlib.example.ackNackModel.messages.AckNackMessage;
import ikr.simlib.messages.Message;
import ikr.simlib.model.SimNode;
import ikr.simlib.parameters.Parameters;
import ikr.simlib.ports.input.InputPort;
import ikr.simlib.ports.output.SynchronousOutputPort;
import ikr.simlib.random.RandomNumberGenerator;

public class Receiver extends Entity {

   @SuppressWarnings("unused")
   private final InputPort input;
   private final SynchronousOutputPort output;
   private final SynchronousOutputPort feedbackOutput;

   public Receiver(SimNode ownNode, Parameters pars) {
      super(ownNode);

      this.output = new SynchronousOutputPort(this);
      this.feedbackOutput = new SynchronousOutputPort(this);

      final double successProbability = pars.get(this.simNode, "SuccessProbability").asDouble();

      this.input = new InputPort(this) {

         @Override
         protected void handleMessageIndication(Message msg) {
            final Message m = getMessage();
            if (RandomNumberGenerator.getSystemRNG().next() <= successProbability) {
               Receiver.this.output.sendMessage(m);
               Receiver.this.feedbackOutput.sendMessage(new AckNackMessage(true));
            } else {
               Receiver.this.feedbackOutput.sendMessage(new AckNackMessage(false));
            }
         }
      };
   }

   public InputPort getInput() {
      return this.input;
   }

   public SynchronousOutputPort getOutput() {
      return this.output;
   }

   public SynchronousOutputPort getFeedbackOutput() {
      return this.feedbackOutput;
   }
}
